/*
 * Copyright (C) 2004 emuWorks
 * http://games.technoplaza.net
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package chtdecoder.tools;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import chtdecoder.datastructures.AbstractCHTCode;
import chtdecoder.datastructures.AbstractCHTCodeList;
import chtdecoder.datastructures.CHTCode;
import chtdecoder.datastructures.CHTCodeIterator;
import chtdecoder.datastructures.CHTCodeList;

import chtdecoder.exceptions.CHTFileException;
import chtdecoder.exceptions.InvalidCHTCodeDataException;

/**
 * A concrete implementation of the CHTFileReader interface.
 *
 * @author John David Ratliff
 * @version 1.1, 09/14/04
 */
class BasicCHTFileReader extends AbstractCHTFileReader {
    private File current;
    private CHTCodeList list;
    private static final BasicCHTFileReader INSTANCE = new BasicCHTFileReader();

    /**
     * Creates a new BasicCHTFileReader object.
     */
    private BasicCHTFileReader() {
        // this is a singleton
        // allow no outside instantation
    }

    /**
     * Gets the singleton instance of this CHTFileReader.
     *
     * @return The singleton instance.
     */
    protected static BasicCHTFileReader getInstance() {
        return INSTANCE;
    }

    /**
     * Reads a .CHT file.
     *
     * @param chtFile The .CHT file to read.
     *
     * @throws CHTFileException if a problem occurs while reading the .CHT
     *         file.
     */
    protected void readFile(File chtFile) {
        this.current = chtFile;
        this.list = AbstractCHTCodeList.createList();

        try {
            FileInputStream input = new FileInputStream(chtFile);

            while (input.available() >= 28) {
                byte[] rawdata = new byte[28];
                input.read(rawdata, 0, 28);

                CHTCode code = AbstractCHTCode.parseCHTCode(rawdata);
                list.add(code);
            }

            input.close();
        } catch (InvalidCHTCodeDataException e) {
            throw new CHTFileException("Invalid CHT Code data encountered!");
        } catch (FileNotFoundException e) {
            throw new CHTFileException("File '" + chtFile.getName() +
                                       "' not found!");
        } catch (IOException e) {
            throw new CHTFileException("IOException: " + e.getMessage());
        }
    }

    /**
     * Returns an iterator for the CHTCodes read from the .CHT file.
     *
     * @return The iterator.
     */
    public CHTCodeIterator iterator() {
        return this.list.iterator();
    }

    /**
     * Returns a String representation of this CHTFileReader.
     *
     * @return A String representation.
     */
    public String toString() {
        return "BasicCHTFileReader[file=" + current + "]";
    }
}
